//============================================================================
// Name        : BDD_IndepSet.cpp
//============================================================================

#include <iostream>
#include <vector>
#include "indepset_instance.hpp"
#include "indepset_bdd.hpp"

using namespace std;

int main(int argc, char* argv[]) {

	// TODO: add parameters
	if (argc < 3) {
		cout << "\nUsage: indepset [graph file] [BDD width]\n\n";
		cout << "\twhere: \n";
		cout << "\t\tgraph file: graph in DIMACS format\n";
		cout << "\t\tBDD width: BDD width at the root node\n";
		cout << endl;
		exit(1);
	}

	// generate relaxation
	int max_width = atoi(argv[2]);
	//IndepSetBDD indepset_bdd(argv[1], max_width);
	// DDX10:
	IndepSetBDD indepset_bdd(0, max_width, argv[1], NULL, 0);


	// ------------------------------------------------------
	// Initial root branching node and bounds
	// ------------------------------------------------------

	// Create root state
	vector<int> root_state( indepset_bdd.inst->graph->n_vertices );
	for (size_t i = 0; i < root_state.size(); ++i) {
		root_state[i] = i;
	}

	// create branching node
	BranchNode* root_branch_node = new BranchNode(root_state, 0);
	root_branch_node->relax_ub = INF;

	// global upper and lower bounds
	double global_lb = 0;
	double global_ub = INF;


	// ------------------------------------------------------
	// Search
	// TODO: apply primal heuristic
	// ------------------------------------------------------

	// initialize branch node queue with root node
	BranchNodeQueue branch_node_queue;
	branch_node_queue.push( root_branch_node );

	// number of nodes explored
	long int nodes_explored = 0;

	// repeat until all search space is explored
	while (!branch_node_queue.empty()) {

		// take branch node from the top
		BranchNode* branch_node = branch_node_queue.top();
		branch_node_queue.pop();

		// update statistics
		nodes_explored++;

		if (nodes_explored % 5 == 0 ) {
			cout << "Explored = " << nodes_explored;
			cout << "\t\tTo Explore = " << branch_node_queue.size();
			cout << "\t\tLB = " << global_lb;
			cout << "\t\tUB = " << global_ub;
			cout << "\t\tgap = " << ((global_ub - global_lb) / global_ub)*100 << "\%";
			cout << endl;
		}

		// explores node if not pruned due to global lower bound
		if (branch_node->relax_ub > global_lb) {

			// explore node
			double ub = indepset_bdd.generate_relaxation(
					branch_node->state,
					branch_node->longest_path);

			// if BDD is exact, update global lower bound
			if (indepset_bdd.is_BDD_exact()) {

				// if global upper bound improved, communicate this to BDDs
				if (global_lb < ub) {
					global_lb = ub;
					indepset_bdd.set_new_lb(global_lb);
				}
				assert( global_lb <= global_ub );
			}

			// add open nodes to pool
			indepset_bdd.add_branch_nodes_pool(branch_node_queue);
		}

		// erase branch node
		delete branch_node;

		// update global upper bound
		if (!branch_node_queue.empty()) {
			global_ub = std::min(global_ub, branch_node_queue.top()->relax_ub);
		}

		// if bounds match, optimum has been found
		if (global_lb == global_ub) {
			break;
		}
	}

	cout << endl;
	cout << "Lower bound = " << global_lb << endl;
	cout << "Upper bound = " << global_ub << endl;

	return 0;
}



