// -----------------------------------------------------------------
// Licensed Materials - Property of IBM
//
// DDX10
//
// (C) Copyright IBM Corporation 2013.  All Rights Reserved.
//
// Authors: Andre Cire, Ashish Sabharwal, Horst Samulowitz,
//          Vijay Saraswat
// -----------------------------------------------------------------


#ifndef DDX10__DD_IndepSetBDD_h
#define DDX10__DD_IndepSetBDD_h

#include "../../DDX10__DD.h"

#include "../c++/IndepSetBDD/src/indepset_bdd.hpp"


// Native c++ code for the x10 class DD_IndepSetBDD
// Creates a DD solver object
class DDX10__DD_IndepSetBDD : public DDX10__DD {
public:
    RTT_H_DECLS_CLASS

    // a specialized pointer
    IndepSetBDD       * dd_IndsepSetBDD;

    // constructor
    DDX10__DD_IndepSetBDD (
            x10_int placeID,
            x10_int ddWidth,
            x10::lang::String* instanceName) :
                DDX10__DD(instanceName->c_str()) {
        // create the DD solver object
        dd = new IndepSetBDD(placeID, ddWidth, instanceName->c_str(), this);

        // store a specialized pointer to this object
        dd_IndsepSetBDD = dynamic_cast<IndepSetBDD*>(dd);

        // print instance information if this is place 0
        if (placeID == 0)
            printInstanceInfo();
    }

    // specialize base class method for printing instance information
    void printInstanceInfo(void) const {
        if (dd == NULL) {
            printf("WARNING: instance information can be printed only after a DD object has been created. \n");
            return;
        }
        DDX10__DD::printInstanceInfo();
        printf("c   number of vertices        : %d\n", dd_IndsepSetBDD->inst->graph->n_vertices);
        printf("c   number of edges           : %d\n", dd_IndsepSetBDD->inst->graph->n_edges);
    }

    // destructor
    virtual ~DDX10__DD_IndepSetBDD() { }

    // _make for X10
    static DDX10__DD_IndepSetBDD* _make(
            x10_int placeID,
            x10_int ddWidth,
            x10::lang::String* instanceName) {
        DDX10__DD_IndepSetBDD* self =
            new (x10aux::alloc<DDX10__DD_IndepSetBDD>()) DDX10__DD_IndepSetBDD(placeID, ddWidth, instanceName);
        return self;
    }
};


#endif
